 /*
  multiplex pattern for Dominoduino Uni (Titchaino uni)
    by ikkei 2015.04.04
    add acceleration sensor  2015.6.6
    reviewed 2018,02.03
    add auto adjust acceleration sensor 2018.04.17
    Applied to AVR64DD28  2024.04.30
*/

#include <arduino.h>
#include <ikkeiplexing.h>
// #include <EEPROM.h>
// #include <avr/sleep.h>
#include "iFont.h"

#include "complex_pattern.h"
#include "heartbeat.h"
#include "random_fade.h"
#include "wave_animation.h"
#include "scroll_text.h"
#include "bounce_balls.h"
#include "fountain.h"
#include "gravity.h"
#include "Tetris.h"
#include "timer.h"


static ikkeiplexing iLedSign;
static iFont iFont;

#define FRAME  0x01
#define TENMS  0x02

static byte icount = 0;
static int sleep_count;
#define SLEEPTIME 0 // no sleep
//#define SLEEPTIME 7500 // 16ms x 7500 = 2 min

byte event = 0;
byte swlvl = 0;
static byte state;
#define SW_STATE 0
#define SW_PEDGE 1
#define SW_NEDGE 2
#define SW_OFF 0
#define SW_1   1
#define SW_2   2

static byte mode_num;
void mode_setup(byte num) {
  icount++;
  if (icount >= 20) {
    icount = 0;
    state = num;
    switch (num) {
      case 1: // space invaders comlex pattern
        complex_pattern_init();
        break;
      case 2: // heart beat
        heartbeat_init();
        break;
      case 3: // random fade
        random_fade_init();
        break;
      case 4: // wave
        wave_animation_init();
        break;
      case 5: // scroll text "Titchaino"
        scroll_text_init();
        break;
      case 6: //
        bounce_balls_init();
        break;
      case 7: //
        fountain_init();
        break;
      case 8: //
        gravity_init();
        break;
    }
  }
}

void display_number(byte num) {
  iLedSign.Clear();
  byte d = iFont.Width('0' + num);
  for (byte i = 0; i < d; i++) {
    iLedSign.Vwrite(i + 7 - d / 2, iFont.Data('0' + num, i));
  }
}


uint8_t smode;

void setup() {
  Serial.begin(115200);
  Serial.println("start");
  iLedSign.Init(DOMINODUINO, SINGLE_BUFFER);

  delay(100);
  smode = iLedSign.GetSW(SW_STATE);

  if (smode == SW_OFF){
    // mode_num = EEPROM.read(0);
    mode_num = 1;
    // if (mode_num > 8 || mode_num == 0) {
    //   mode_num = 1;
    //   EEPROM.write(0, mode_num);
    // }
    mode_setup(mode_num);
  } else if (smode == SW_1){
    tetris_init();
  } else if (smode == SW_2){
    timer_init();
  }
}

void loop() {
  if (smode == SW_OFF){
    if (iLedSign.GetPeriod(FRAME)){
      event = iLedSign.GetSW(SW_PEDGE);
      if (event == 1) {
        mode_num++;
        if (mode_num > 8 || mode_num == 0) {
          mode_num = 1;
        }
        // EEPROM.write(0, mode_num);
        display_number(mode_num);
        icount = 0;
        state = 0;
        sleep_count = SLEEPTIME;
      }
      switch (state) {
        case 0:
          mode_setup(mode_num);
          break;
        case 1:
          invader();
          break;
        case 2:
          heartbeat();
          break;
        case 3:
          random_fade();
          break;
        case 4:
          wave_animation();
          break;
        case 5:
          scroll_text();
          break;
        case 6:
          bounce_balls();
          break;
        case 7:
          fountain();
          break;
        case 8:
          gravity();
          break;
      }
    }
  } else if (smode == SW_1){
    tetris();
  } else if (smode == SW_2){
    timer();
  }
}
